package com.jeeplus.assess.reimbursement.reimbursementInfo.controller;

import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.jeeplus.assess.projectRecords.Utils.EasyPoiUtil;
import com.jeeplus.assess.reimbursement.reimbursementInfo.domain.ReimbursementFileSupplement;
import com.jeeplus.assess.reimbursement.reimbursementInfo.domain.ReimbursementInfoUpHi;
import com.jeeplus.assess.reimbursement.reimbursementInfo.service.dto.*;
import com.jeeplus.common.excel.ExcelOptions;
import com.jeeplus.common.excel.ExportMode;
import com.jeeplus.common.utils.ResponseUtil;
import com.jeeplus.assess.program.configuration.projectList.domain.ProgramProjectListInfo;
import com.jeeplus.assess.reimbursement.reimbursementInfo.domain.ReimbursementInfo;
import com.jeeplus.assess.reimbursement.reimbursementInfo.service.ReimbursementInfoService;
import com.jeeplus.logging.annotation.ApiLog;
import com.jeeplus.sys.feign.IDictApi;
import com.jeeplus.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

@RestController
@Api(tags ="报销申请")
@RequestMapping(value = "/reimbursement/info")
public class ReimbursementInfoController {

    @Resource
    private ReimbursementInfoService service;

    /**
     * 列表查询
     * @param dto
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/list")
    public ResponseEntity<IPage<RetureListDto>> list(Page<RetureListDto> page, QueryListDto dto) throws Exception{
        IPage<RetureListDto> iPage = service.list(page, dto);
        return ResponseEntity.ok(iPage);
    }

    /**
     * 新增/修改
     * @param dto
     * @return
     */
    @ApiOperation(value = "新增、修改")
    @PostMapping("/save")
    public ResponseEntity<String> save(@RequestBody SaveInfoDto dto) throws Exception{
        ReimbursementInfo s = service.save(dto);
        return ResponseUtil.newInstance().add("businessTable", "reimbursement_info")
                .add("businessId", s.getId())
                .add("no", s.getNo())
                .ok ("操作成功");
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @ApiOperation(value = "删除")
    @GetMapping("/remove")
    public ResponseEntity<String> remove(@RequestParam String id){
        String s = service.remove(id);
        return ResponseEntity.ok(s);
    }

    /**
     * 查询
     * @param id
     * @return
     */
    @ApiOperation(value = "查询")
    @GetMapping("/findById")
    public ResponseEntity<SaveInfoDto> findById(@RequestParam("id") String id) throws Exception{
        SaveInfoDto dto = service.findById(id);
        return ResponseEntity.ok(dto);
    }

    @ApiOperation(value = "发票号判断")
    @GetMapping("/checkNumber")
    public ResponseEntity<Boolean> checkNumber(@RequestParam("number") String number) {
        Boolean is = service.checkNumber(number);
        return ResponseEntity.ok(is);
    }

    /**
     * 根据id修改状态status
     */
    @ApiOperation(value = "根据id修改状态status")
    @PostMapping(value = "updateStatusById")
    public void updateStatusById(@RequestBody RetureListDto dto) {
        service.updateStatusById(dto);
    }

    @ApiOperation(value = "用户树形")
    @GetMapping(value = "/userTree")
    public ResponseEntity<List<TreeUserDto>> userTree(@RequestParam String name) {
        List<TreeUserDto> list = service.userTree(name);
        return ResponseEntity.ok(list);
    }
    @ApiOperation(value = "用户树形")
    @GetMapping(value = "/userTree2")
    public ResponseEntity<List<TreeUserDto>> userTree2(@RequestParam String name) {
        List<TreeUserDto> list = service.userTree2(name);
        return ResponseEntity.ok(list);
    }
    @ApiOperation(value = "用户树形")
    @GetMapping(value = "/userTreeSelectBySignatureScribe")
    public ResponseEntity<List<TreeUserDto>> userTreeSelectBySignatureScribe(@RequestParam String name) {
        List<TreeUserDto> list = service.userTreeSelectBySignatureScribe(name);
        return ResponseEntity.ok(list);
    }


    @ApiLog(value = "评估报销数据")
    @GetMapping("exportFile")
    @ApiOperation(value = "评估报销数据")
    public void exportFile(QueryListDto cwDTO, Page <RetureListDto> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = options.getFilename();
        String sheetName = options.getSheetName();
        List<RetureListDto> result = new ArrayList<>();
        if ( ExportMode.current.equals ( options.getMode() ) ) {
            result = service.list (page,cwDTO).getRecords();
        } else if (ExportMode.selected.equals ( options.getMode() )) {
            result = service.list (page,cwDTO).getRecords().stream ( ).filter ( item ->
                    options.getSelectIds ( ).contains ( item.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize (-1);
            page.setCurrent (0);
            result = service.list (page,cwDTO).getRecords();
        }

        if(result.size()>0){
            //获取字典数据
            String statusDatas = SpringUtil.getBean ( IDictApi.class ).getDictListMapByDict ("status");
            Map<String,Object> statusValueDTOs = JSON.parseObject(statusDatas, new TypeReference<Map<String,Object>>() {});

            String cwReimbursementSourceTypeDatas = SpringUtil.getBean ( IDictApi.class ).getDictListMapByDict ("pg_reimbursement_source_type");
            Map<String,Object> cwReimbursementSourceTypeValueDTOs = JSON.parseObject(cwReimbursementSourceTypeDatas, new TypeReference<Map<String,Object>>() {});

            String paymentStatusDatas = SpringUtil.getBean ( IDictApi.class ).getDictListMapByDict ("payment_status");
            Map<String,Object> paymentStatusValueDTOs = JSON.parseObject(paymentStatusDatas, new TypeReference<Map<String,Object>>() {});

            for (RetureListDto info : result) {

                if(org.apache.commons.lang3.StringUtils.isNotBlank(info.getType())){
                    for (String key : statusValueDTOs.keySet()) {
                        if(info.getType().equals(key)){
                            info.setType(String.valueOf(statusValueDTOs.get(key)));
                            break;
                        }
                    }
                }
                if(org.apache.commons.lang3.StringUtils.isNotBlank(info.getSourceType())){
                    for (String key : cwReimbursementSourceTypeValueDTOs.keySet()) {
                        if(info.getSourceType().equals(key)){
                            info.setSourceType(String.valueOf(cwReimbursementSourceTypeValueDTOs.get(key)));
                            break;
                        }
                    }
                }
                if(org.apache.commons.lang3.StringUtils.isNotBlank(info.getPaymentStatus())){
                    for (String key : paymentStatusValueDTOs.keySet()) {
                        if(info.getPaymentStatus().equals(key)){
                            info.setPaymentStatus(String.valueOf(paymentStatusValueDTOs.get(key)));
                            break;
                        }
                    }
                }else {
                    info.setPaymentStatus("未付款");
                }
            }
        }



        EasyPoiUtil.exportExcel ( result, sheetName,  sheetName, RetureListDto.class, fileName, response );

    }


    @ApiLog(value = "评估电子发票报销数据")
    @GetMapping("exportInvoiceReimbursementFile")
    @ApiOperation(value = "评估电子发票报销数据")
    public void exportInvoiceReimbursementFile(QueryListDto cwDTO, Page <RetureListDto> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = "评估数电发票报销数据表";
        String sheetName = "评估数电发票报销列表";
        List<RetureListDto> result = new ArrayList<>();
        if ( ExportMode.current.equals ( options.getMode() ) ) {
            result = service.list (page,cwDTO).getRecords();
        } else if (ExportMode.selected.equals ( options.getMode() )) {
            result = service.list (page,cwDTO).getRecords().stream ( ).filter ( item ->
                    options.getSelectIds ( ).contains ( item.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize (-1);
            page.setCurrent (0);
            if(StringUtils.isBlank(cwDTO.getReimbursementType())){
                cwDTO.setReimbursementType("1");
            }
            result = service.list (page,cwDTO).getRecords();
        }
        List<String> idList = Lists.newArrayList();
        for (RetureListDto listDto : result) {
            idList.add(listDto.getId());
        }
        List<ReimbursementAmountInfoDTO> invoiceReimbursementAmountList = Lists.newArrayList();
        if(idList.size()>0){
            invoiceReimbursementAmountList = service.getInvoiceReimbursementAmountList(idList);
        }
        for (int i = 1;i <=invoiceReimbursementAmountList.size();i ++){
            ReimbursementAmountInfoDTO reimbursementAmountInfoDTO = invoiceReimbursementAmountList.get(i - 1);
            reimbursementAmountInfoDTO.setSerialNumber(i);
        }

        EasyPoiUtil.exportExcel ( invoiceReimbursementAmountList, sheetName,  sheetName, ReimbursementAmountInfoDTO.class, fileName, response );

    }

    @ApiOperation(value = "关联报销")
    @GetMapping(value = "/relationReimbursementList")
    public ResponseEntity<IPage<RetureListDto>> relationReimbursementList(Page<RetureListDto> page, @RequestParam("id") String id) {
        IPage<RetureListDto> listDtoIPage = service.relationReimbursementList(page, id);
        return ResponseEntity.ok(listDtoIPage);
    }

    @ApiOperation(value = "报销审核")
    @PostMapping(value = "/auditFunc")
    public Map<String,Object> auditFunc(@RequestBody SaveInfoDto saveInfoDto) {
        Map<String, Object> res = service.auditFunc(saveInfoDto);
        return res;
    }

    @ApiOperation(value = "列表查询")
    @GetMapping("/queryByProIds")
    public ResponseEntity<List<ProgramProjectListInfo>> queryByProIds(String ids){
        List<ProgramProjectListInfo> cwProjectRecords = service.queryByProIds(ids);
        return ResponseEntity.ok(cwProjectRecords);
    }

    /**
     * 根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据发票号查询是否已经或正在报销")
    @GetMapping("getEffectiveDataByInvoiceNumber")
    @ApiOperation(value = "根据发票号查询是否已经或正在报销")
    public Map<String,Object> getEffectiveDataByInvoiceNumber(String invoiceNumber,String id){
        Map map = new HashMap();
        if(org.apache.commons.lang3.StringUtils.isBlank(invoiceNumber)){
            map.put("success",false);
            map.put("message","发票号不可为空");
        }else{
            Integer count = service.getEffectiveDataByInvoiceNumber(invoiceNumber,id);
            if(count>0){
                map.put("success",false);
                map.put("message","发票号：" + invoiceNumber + " 已发起或完成报销，请勿重复报销");
            }else{
                map.put("success",true);
            }
        }
        return map;
    }


    /**
     * 提交时根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据发票号查询是否已经或正在报销")
    @GetMapping("getEffectiveDataByNumbers")
    @ApiOperation(value = "根据发票号查询是否已经或正在报销")
    public Map<String,Object> getEffectiveDataByNumbers(String invoiceNumber,String id){
        Map map = new HashMap();
        map.put("success",true);
        map.put("message","数电发票正常");
        if(org.apache.commons.lang3.StringUtils.isNotBlank(invoiceNumber)){
            String[] array = invoiceNumber.split(",");
            // 将数组转换为 List
            List<String> numberList = Arrays.asList(array);
            Set<String> set = new HashSet<>();
            if(numberList.size()>0){
                List<String> repetitionMessageList = service.getEffectiveDataByNumbers(numberList,id);
                for (String vatTax : repetitionMessageList) {
                    set.add(vatTax);
                }
            }
            ArrayList<String> strings = Lists.newArrayList(set);
            String result = String.join(",", strings);
            if(result.length()>0){
                map.put("success",false);
                map.put("message","数电发票编号为：" + result + "已经发起或已完成报销，无法重复报销");
            }

        }
        return map;
    }


    /**
     * 提交时根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据发票号查询是否已经或正在报销")
    @GetMapping("getEffectiveDataByNumbers2")
    @ApiOperation(value = "根据发票号查询是否已经或正在报销")
    public Map<String,Object> getEffectiveDataByNumbers2(String invoiceNumber,String id){
        Map map = new HashMap();
        map.put("success",true);
        map.put("message","数电发票正常");
        if(org.apache.commons.lang3.StringUtils.isNotBlank(invoiceNumber)){
            String[] array = invoiceNumber.split(",");
            // 将数组转换为 List
            List<String> numberList = Arrays.asList(array);
            Set<String> set = new HashSet<>();
            if(numberList.size()>0){
                List<String> repetitionMessageList = service.getEffectiveDataByNumbers2(numberList,id);
                for (String vatTax : repetitionMessageList) {
                    set.add(vatTax);
                }
            }
            ArrayList<String> strings = Lists.newArrayList(set);
            String result = String.join(",", strings);
            if(result.length()>0){
                map.put("success",false);
                map.put("message","数电发票编号为：" + result + "已经发起或已完成报销，无法重复报销");
            }

        }
        return map;
    }


    /**
     * 根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据报销id和发票编号删除数据记录")
    @PostMapping("deleteByIdAndNumber")
    @ApiOperation(value = "根据报销id和发票编号删除数据记录")
    public Map<String,Object> deleteByIdAndNumber(String id,String invoiceNumber){
        Map map = new HashMap();
        if(StringUtils.isBlank(invoiceNumber)){
            map.put("success",false);
            map.put("message","发票号不可为空");
        }else if(StringUtils.isBlank(id)){
            map.put("success",false);
            map.put("message","报销信息未选择");
        }else{
            Integer count = service.deleteByIdAndNumber(invoiceNumber,id);
            map.put("success",true);
        }
        return map;
    }

    /**
     * 查询有无历史数据
     * @param id
     * @return
     */
    @ApiOperation(value = "查询有无历史数据")
    @GetMapping("/findHistory")
    public ResponseEntity<List<ReimbursementInfoUpHi>> findHistory(@RequestParam String id) throws Exception{
        List<ReimbursementInfoUpHi> list=service.findHistory(id);
        return ResponseEntity.ok(list);
    }

    /**
     * 根据id查询历史数据
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id查询历史数据")
    @GetMapping("/findHiById")
    public ResponseEntity<SaveInfoDto> findHiById(@RequestParam String id) throws Exception{
        SaveInfoDto list=service.findHiById(id);
        return ResponseEntity.ok(list);
    }

    /**
     * 修改付款状态
     * @param
     * @return
     */
    @ApiOperation(value = "修改付款状态")
    @PostMapping("/updatePaymentStatus")
    public String updatePaymentStatus(@RequestBody RetureListDto dto) {
        String data=service.updatePaymentStatus(dto.getId(),dto.getPaymentStatus());
        return data;
    }

    /**
     * 查询文件补充
     * @param id
     * @return
     */
    @ApiOperation(value = "查询文件补充")
    @GetMapping("/findFileById")
    public ResponseEntity<SaveInfoDto> findFileById(@RequestParam String id) throws Exception{
        SaveInfoDto dto = service.findFileById(id);
        return ResponseEntity.ok(dto);
    }

    /**
     * 新增/修改 文件补充
     * @param dto
     * @return
     */
    @ApiOperation(value = "新增、修改 文件补充")
    @PostMapping("/saveFile")
    public ResponseEntity<String> saveFile(@RequestBody SaveInfoDto dto) throws Exception{
        ReimbursementFileSupplement s = service.saveFile(dto);
        return ResponseUtil.newInstance().add("businessTable", "reimbursement_file_supplement")
                .add("businessId", s.getId())
                .add("no", s.getNo())
                .add("name",s.getName())
                .ok ("操作成功");
    }

    /**
     * 根据id修改文件补充状态status
     */
    @ApiOperation(value = "根据id修改文件补充状态status")
    @PostMapping(value = "updateFileStatusById")
    public void updateFileStatusById(@RequestBody RetureListDto dto) {
        service.updateFileStatusById(dto);
    }



    /**
     * 修改数电发票报销数据(对老数据进行调整)
     */
    @ApiOperation(value = "修改数电发票报销数据(对老数据进行调整)")
    @GetMapping(value = "updateOldData")
    public void updateOldData(){
        service.updateOldData();
    }
}
