package com.jeeplus.ccpm.approvalType.controller;
import com.jeeplus.ccpm.approvalType.domain.CcpmReimbursementTypeInfo;
import com.jeeplus.ccpm.approvalType.mapper.CcpmReimbursementTypeMapper;
import com.jeeplus.ccpm.approvalType.service.CcpmReimbursementTypeForTreeDataService;
import com.jeeplus.ccpm.approvalType.service.CcpmReimbursementTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author: 王强
 * @create: 2022-11-25 09:18
 **/
@RestController
@Api(tags ="项目-报销类型管理")
@RequestMapping(value = "/ccpmReimbursement/type")
public class CcpmReimbursementTypeController {

    @Resource
    private CcpmReimbursementTypeService service;

    @Resource
    private CcpmReimbursementTypeForTreeDataService treeService;

    @Resource
    private CcpmReimbursementTypeMapper mapper;

    /**
     * 列表查询
     * @param info
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/list")
    public ResponseEntity<List<CcpmReimbursementTypeInfo>> list(CcpmReimbursementTypeInfo info) {
        List<CcpmReimbursementTypeInfo> list = service.list(info);
        return ResponseEntity.ok(list);
    }

    /**
     * 列表查询
     * @param info
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/bxList")
    public ResponseEntity<List<CcpmReimbursementTypeInfo>> bxList(CcpmReimbursementTypeInfo info) {
        List<CcpmReimbursementTypeInfo> list = service.bxList(info);
        return ResponseEntity.ok(list);
    }

    /**
     * 采购类型列表查询
     * @param info
     * @return
     */
    @ApiOperation(value = "采购类型列表查询")
    @GetMapping("/cgList")
    public ResponseEntity<List<CcpmReimbursementTypeInfo>> cgList(CcpmReimbursementTypeInfo info) {
        List<CcpmReimbursementTypeInfo> list = service.cgList(info);
        return ResponseEntity.ok(list);
    }

    /**
     * 新增/修改
     * @param info
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @PostMapping("/save")
    public ResponseEntity<String> save(@RequestBody CcpmReimbursementTypeInfo info) {
        String s = service.save(info);
        return ResponseEntity.ok(s);
    }

    /**
     * 根据id查询
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id查询")
    @GetMapping("/findById")
    public ResponseEntity<CcpmReimbursementTypeInfo> findById(String id) {
        CcpmReimbursementTypeInfo info = mapper.selectById(id);
        return ResponseEntity.ok(info);
    }

    /**
     * 根据id删除
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id删除")
    @GetMapping("/deleteById")
    public ResponseEntity<String> deleteById(String id) {
        mapper.deleteById(id);
        return ResponseEntity.ok("操作完成");
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/treeData")
    public ResponseEntity<List<CcpmReimbursementTypeInfo>> treeData(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<CcpmReimbursementTypeInfo> infos = treeService.treeDataForType(extId, type);
        return ResponseEntity.ok(infos);
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/treeData1")
    public ResponseEntity<List<CcpmReimbursementTypeInfo>> treeData1(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<CcpmReimbursementTypeInfo> infos = treeService.treeDataForType1(extId, type);
        return ResponseEntity.ok(infos);
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/summaryTreeData")
    public ResponseEntity<List<CcpmReimbursementTypeInfo>> summaryTreeData(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<CcpmReimbursementTypeInfo> infos = treeService.summaryTreeData(extId, type);
        return ResponseEntity.ok(infos);
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/treeData2")
    public ResponseEntity<List<CcpmReimbursementTypeInfo>> treeData2(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<CcpmReimbursementTypeInfo> infos = treeService.treeDataForType2(extId, type);
        return ResponseEntity.ok(infos);
    }

}
