package com.jeeplus.consultancy.approvalInfo.controller;

import cn.hutool.extra.spring.SpringUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.Lists;
import com.jeeplus.common.excel.ExcelOptions;
import com.jeeplus.common.excel.ExportMode;
import com.jeeplus.common.utils.ResponseUtil;
import com.jeeplus.consultancy.approvalInfo.domain.ConsultancyProgramProjectListInfo;
import com.jeeplus.consultancy.approvalInfo.domain.ConsultancyReimbursementFileSupplement;
import com.jeeplus.consultancy.approvalInfo.domain.ConsultancyReimbursementInfo;
import com.jeeplus.consultancy.approvalInfo.domain.ConsultancyReimbursementInfoUpHi;
import com.jeeplus.consultancy.approvalInfo.service.ConsultancyReimbursementInfoService;
import com.jeeplus.consultancy.approvalInfo.service.dto.*;
import com.jeeplus.consultancy.util.ConsultancyEasyPoiUtil;
import com.jeeplus.flowable.feign.IFlowableApi;
import com.jeeplus.logging.annotation.ApiLog;
import com.jeeplus.logging.constant.enums.LogTypeEnum;
import com.jeeplus.sys.feign.IDictApi;
import com.jeeplus.utils.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author: 桑文巍
 * @create: 2022-11-24 16:44
 **/
@RestController
@Api(tags ="咨询-报销申请")
@RequestMapping(value = "/consultancyReimbursement/info")
public class ConsultancyReimbursementInfoController {

    @Resource
    private ConsultancyReimbursementInfoService service;

    @Resource
    private IFlowableApi flowTaskService;

    /**
     * 报告号查询
     * @param dto
     * @return
     */
    @ApiOperation(value = "报告号查询")
    @GetMapping("/reportNoList")
    public ResponseEntity<IPage<ConsultancyReportNoDto>> reportNoList(Page<ConsultancyReportNoDto> page, ConsultancyReportNoDto dto) throws Exception{
        IPage<ConsultancyReportNoDto> iPage = service.getReportNoList(page, dto);
        return ResponseEntity.ok(iPage);
    }

    /**
     * 列表查询
     * @param dto/reimbursementApproval/type/bxList
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/list")
    public ResponseEntity<IPage<ConsultancyRetureListDto>> list(Page<ConsultancyRetureListDto> page, ConsultancyQueryListDto dto) throws Exception{
        IPage<ConsultancyRetureListDto> iPage = service.list(page, dto);
        iPage.getRecords().stream().forEach(i -> {
            if (StringUtils.isNotBlank(i.getProcInsId()) && StringUtils.isNotBlank(i.getTaskId())) {
                i.setAuditUserIds(flowTaskService.getTaskAuditUsers(i.getTaskId()));  // 获取数据审核人
                String currentTaskName = flowTaskService.getCurrentTaskName(i.getProcInsId());
                if (i.getType().equals("2") && StringUtils.isNotBlank(currentTaskName) && !currentTaskName.contains("综合管理部主任") && !currentTaskName.contains("部门主任")){
                    i.setApplyFile(true);
                }
            }
            //文件补充
            if (StringUtils.isNotBlank(i.getTaskIdFile()) && org.apache.commons.lang3.StringUtils.isNotBlank(i.getFileStatus())) {
                if ("2".equals(i.getFileStatus()) || "4".equals(i.getFileStatus())) { // “审核中”的数据要获取数据审核人
                    i.setAuditFileUserIds(flowTaskService.getTaskAuditUsers(i.getTaskIdFile()));  // 获取数据审核人
                }
            }
        });
        return ResponseEntity.ok(iPage);
    }

    /**
     * 项目登记列表
     * @param info
     * @param page
     * @return
     * @throws Exception
     */
    @ApiOperation(value = "项目登记列表")
    @GetMapping(value = "/projectList")
    public ResponseEntity<IPage<ConsultancyProgramProjectListInfo>> projectList(ConsultancyProgramProjectListInfo info, Page<ConsultancyProgramProjectListInfo> page) throws Exception {
        IPage<ConsultancyProgramProjectListInfo> list = service.projectList(page, info, "1");
        return ResponseEntity.ok (list);
    }

    /**
     * 新增/修改
     * @param dto
     * @return
     */
    @ApiOperation(value = "新增、修改")
    @PostMapping("/save")
    public ResponseEntity<String> save(@RequestBody ConsultancySaveInfoDto dto) throws Exception{
        ConsultancyReimbursementInfo s = service.save(dto);
        return ResponseUtil.newInstance().add("businessTable", "consultancy_reimbursement_info")
                .add("businessId", s.getId())
                .add("no", s.getNo())
                .ok ("操作成功");
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @ApiOperation(value = "删除")
    @GetMapping("/remove")
    public ResponseEntity<String> remove(@RequestParam String id){
        String s = service.remove(id);
        return ResponseEntity.ok(s);
    }

    /**
     * 查询
     * @param id
     * @return
     */
    @ApiOperation(value = "查询")
    @GetMapping("/findById")
    public ResponseEntity<ConsultancySaveInfoDto> findById(@RequestParam String id) throws Exception{
        ConsultancySaveInfoDto dto = service.findById(id);
        return ResponseEntity.ok(dto);
    }

    @ApiOperation(value = "发票号判断")
    @GetMapping("/checkNumber")
    public ResponseEntity<Boolean> checkNumber(@RequestParam String number) {
        Boolean is = service.checkNumber(number);
        return ResponseEntity.ok(is);
    }

    /**
     * 根据id修改状态status
     */
    @ApiOperation(value = "根据id修改状态status")
    @PostMapping(value = "updateStatusById")
    public void updateStatusById(@RequestBody ConsultancyRetureListDto dto) {
        service.updateStatusById(dto);
    }

    @ApiOperation(value = "用户树形")
    @GetMapping(value = "/userTree")
    public ResponseEntity<List<ConsultancyTreeUserDto>> userTree(@RequestParam String name) {
        List<ConsultancyTreeUserDto> list = service.userTree(name);
        return ResponseEntity.ok(list);
    }



    @ApiLog(value = "项目报销数据", type = LogTypeEnum.EXPORT)
    @GetMapping("exportFile")
    @ApiOperation(value = "项目报销数据")
    public void exportFile(ConsultancyQueryListDto cwFinanceInvoiceDTO, Page <ConsultancyRetureListDto> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = options.getFilename();
        String sheetName = options.getSheetName();
        List<ConsultancyRetureListDto> result = new ArrayList<>();
        if ( ExportMode.current.equals ( options.getMode() ) ) {
            result = service.list (page,cwFinanceInvoiceDTO).getRecords();
        } else if (ExportMode.selected.equals ( options.getMode() )) {
            result = service.list (page,cwFinanceInvoiceDTO).getRecords().stream ( ).filter ( item ->
                    options.getSelectIds ( ).contains ( item.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize (-1);
            page.setCurrent (0);
            result = service.list (page,cwFinanceInvoiceDTO).getRecords();
        }
        if(result.size()>0){
            //获取字典数据
            String statusDatas = SpringUtil.getBean ( IDictApi.class ).getDictListMapByDict ("status");
            Map<String,Object> statusValueDTOs = JSON.parseObject(statusDatas, new TypeReference<Map<String,Object>>() {});

            String cwReimbursementSourceTypeDatas = SpringUtil.getBean ( IDictApi.class ).getDictListMapByDict ("cw_reimbursement_source_type");
            Map<String,Object> cwReimbursementSourceTypeValueDTOs = JSON.parseObject(cwReimbursementSourceTypeDatas, new TypeReference<Map<String,Object>>() {});

            String paymentStatusDatas = SpringUtil.getBean ( IDictApi.class ).getDictListMapByDict ("payment_status");
            Map<String,Object> paymentStatusValueDTOs = JSON.parseObject(paymentStatusDatas, new TypeReference<Map<String,Object>>() {});

            for (ConsultancyRetureListDto info : result) {

                if(org.apache.commons.lang3.StringUtils.isNotBlank(info.getType())){
                    for (String key : statusValueDTOs.keySet()) {
                        if(info.getType().equals(key)){
                            info.setType(String.valueOf(statusValueDTOs.get(key)));
                            break;
                        }
                    }
                }
                if(org.apache.commons.lang3.StringUtils.isNotBlank(info.getSourceType())){
                    for (String key : cwReimbursementSourceTypeValueDTOs.keySet()) {
                        if(info.getSourceType().equals(key)){
                            info.setSourceType(String.valueOf(cwReimbursementSourceTypeValueDTOs.get(key)));
                            break;
                        }
                    }
                }
                if(org.apache.commons.lang3.StringUtils.isNotBlank(info.getPaymentStatus())){
                    for (String key : paymentStatusValueDTOs.keySet()) {
                        if(info.getPaymentStatus().equals(key)){
                            info.setPaymentStatus(String.valueOf(paymentStatusValueDTOs.get(key)));
                            break;
                        }
                    }
                }else {
                    info.setPaymentStatus("未付款");
                }
            }
        }

        ConsultancyEasyPoiUtil.exportExcel ( result, sheetName,  sheetName, ConsultancyRetureListDto.class, fileName, response );

    }

    @ApiLog(value = "项目电子发票报销数据")
    @GetMapping("exportInvoiceReimbursementFile")
    @ApiOperation(value = "项目电子发票报销数据")
    public void exportInvoiceReimbursementFile(ConsultancyQueryListDto cwDTO, Page <ConsultancyRetureListDto> page, ExcelOptions options, HttpServletResponse response) throws Exception {
        String fileName = "项目数电发票报销数据表";
        String sheetName = "项目数电发票报销列表";
        List<ConsultancyRetureListDto> result = new ArrayList<>();
        if (ExportMode.current.equals ( options.getMode() ) ) {
            result = service.list (page,cwDTO).getRecords();
        } else if (ExportMode.selected.equals ( options.getMode() )) {
            result = service.list (page,cwDTO).getRecords().stream ( ).filter ( item ->
                    options.getSelectIds ( ).contains ( item.getId ( ) )
            ).collect ( Collectors.toList ( ) );
        } else {
            page.setSize (-1);
            page.setCurrent (0);
            if(StringUtils.isBlank(cwDTO.getReimbursementType())){
                cwDTO.setReimbursementType("1");
            }
            result = service.list (page,cwDTO).getRecords();
        }
        List<String> idList = Lists.newArrayList();
        for (ConsultancyRetureListDto listDto : result) {
            idList.add(listDto.getId());
        }
        List<ConsultancyReimbursementAmountInfoDTO> invoiceReimbursementAmountList = Lists.newArrayList();
        if(idList.size()>0){
            invoiceReimbursementAmountList = service.getInvoiceReimbursementAmountList(idList);
        }
        for (int i = 1;i <=invoiceReimbursementAmountList.size();i ++){
            ConsultancyReimbursementAmountInfoDTO reimbursementAmountInfoDTO = invoiceReimbursementAmountList.get(i - 1);
            reimbursementAmountInfoDTO.setSerialNumber(i);
        }

        ConsultancyEasyPoiUtil.exportExcel ( invoiceReimbursementAmountList, sheetName,  sheetName, ConsultancyReimbursementAmountInfoDTO.class, fileName, response );

    }



    @ApiOperation(value = "关联报销")
    @GetMapping(value = "/relationReimbursementList")
    public ResponseEntity<IPage<ConsultancyRetureListDto>> relationReimbursementList(Page<ConsultancyRetureListDto> page, @RequestParam("id") String id) {
        IPage<ConsultancyRetureListDto> listDtoIPage = service.relationReimbursementList(page, id);
        return ResponseEntity.ok(listDtoIPage);
    }

    /**
     * 根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据发票号查询是否已经或正在报销")
    @GetMapping("getEffectiveDataByInvoiceNumber")
    @ApiOperation(value = "根据发票号查询是否已经或正在报销")
    public Map<String,Object> getEffectiveDataByInvoiceNumber(String invoiceNumber, String id){
        Map map = new HashMap();
        if(org.apache.commons.lang3.StringUtils.isBlank(invoiceNumber)){
            map.put("success",false);
            map.put("message","发票号不可为空");
        }else{
            Integer count = service.getEffectiveDataByInvoiceNumber(invoiceNumber,id);
            if(count>0){
                map.put("success",false);
                map.put("message","发票号：" + invoiceNumber + " 已发起或完成报销，请勿重复报销");
            }else{
                map.put("success",true);
            }
        }
        return map;
    }
    /**
     * 提交时根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据发票号查询是否已经或正在报销")
    @GetMapping("getEffectiveDataByNumbers")
    @ApiOperation(value = "根据发票号查询是否已经或正在报销")
    public Map<String,Object> getEffectiveDataByNumbers(String invoiceNumber,String id){
        Map map = new HashMap();
        map.put("success",true);
        map.put("message","数电发票正常");
        if(org.apache.commons.lang3.StringUtils.isNotBlank(invoiceNumber)){
            String[] array = invoiceNumber.split(",");
            // 将数组转换为 List
            List<String> numberList = Arrays.asList(array);
            Set<String> set = new HashSet<>();
            if(numberList.size()>0){
                List<String> repetitionMessageList = service.getEffectiveDataByNumbers(numberList,id);
                for (String vatTax : repetitionMessageList) {
                    set.add(vatTax);
                }
            }
            ArrayList<String> strings = Lists.newArrayList(set);
            String result = String.join(",", strings);
            if(result.length()>0){
                map.put("success",false);
                map.put("message","数电发票编号为：" + result + "已经发起或已完成报销，无法重复报销");
            }

        }
        return map;
    }


    /**
     * 提交时根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据发票号查询是否已经或正在报销")
    @GetMapping("getEffectiveDataByNumbers2")
    @ApiOperation(value = "根据发票号查询是否已经或正在报销")
    public Map<String,Object> getEffectiveDataByNumbers2(String invoiceNumber,String id){
        Map map = new HashMap();
        map.put("success",true);
        map.put("message","数电发票正常");
        if(org.apache.commons.lang3.StringUtils.isNotBlank(invoiceNumber)){
            String[] array = invoiceNumber.split(",");
            // 将数组转换为 List
            List<String> numberList = Arrays.asList(array);
            Set<String> set = new HashSet<>();
            if(numberList.size()>0){
                List<String> repetitionMessageList = service.getEffectiveDataByNumbers2(numberList, id);
                for (String vatTax : repetitionMessageList) {
                    set.add(vatTax);
                }
            }
            ArrayList<String> strings = Lists.newArrayList(set);
            String result = String.join(",", strings);
            if(result.length()>0){
                map.put("success",false);
                map.put("message","数电发票编号为：" + result + "已经发起或已完成报销，无法重复报销");
            }

        }
        return map;
    }
    /**
     * 根据发票号查询是否已经或正在报销
     * @return
     */
    @ApiLog(value = "根据报销id和发票编号删除数据记录")
    @PostMapping("deleteByIdAndNumber")
    @ApiOperation(value = "根据报销id和发票编号删除数据记录")
    public Map<String,Object> deleteByIdAndNumber(String id,String invoiceNumber){
        Map map = new HashMap();
        if(StringUtils.isBlank(invoiceNumber)){
            map.put("success",false);
            map.put("message","发票号不可为空");
        }else if(StringUtils.isBlank(id)){
            map.put("success",false);
            map.put("message","报销信息未选择");
        }else{
            Integer count = service.deleteByIdAndNumber(invoiceNumber,id);
            map.put("success",true);
        }
        return map;
    }

    /**
     * 查询有无历史数据
     * @param id
     * @return
     */
    @ApiOperation(value = "查询有无历史数据")
    @GetMapping("/findHistory")
    public ResponseEntity<List<ConsultancyReimbursementInfoUpHi>> findHistory(@RequestParam String id) throws Exception{
        List<ConsultancyReimbursementInfoUpHi> list=service.findHistory(id);
        return ResponseEntity.ok(list);
    }

    /**
     * 根据id查询历史数据
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id查询历史数据")
    @GetMapping("/findHiById")
    public ResponseEntity<ConsultancySaveInfoDto> findHiById(@RequestParam String id) throws Exception{
        ConsultancySaveInfoDto list=service.findHiById(id);
        return ResponseEntity.ok(list);
    }

    /**
     * 修改付款状态
     * @param
     * @return
     */
    @ApiOperation(value = "修改付款状态")
    @PostMapping("/updatePaymentStatus")
    public String updatePaymentStatus(@RequestBody ConsultancyRetureListDto dto) {
        String data=service.updatePaymentStatus(dto.getId(),dto.getPaymentStatus());
        return data;
    }

    /**
     * 查询文件补充
     * @param id
     * @return
     */
    @ApiOperation(value = "查询文件补充")
    @GetMapping("/findFileById")
    public ResponseEntity<ConsultancySaveInfoDto> findFileById(@RequestParam String id) throws Exception{
        ConsultancySaveInfoDto dto = service.findFileById(id);
        return ResponseEntity.ok(dto);
    }

    /**
     * 新增/修改 文件补充
     * @param dto
     * @return
     */
    @ApiOperation(value = "新增、修改 文件补充")
    @PostMapping("/saveFile")
    public ResponseEntity<String> saveFile(@RequestBody ConsultancySaveInfoDto dto) throws Exception{
        ConsultancyReimbursementFileSupplement s = service.saveFile(dto);
        return ResponseUtil.newInstance().add("businessTable", "consultancy_reimbursement_file_supplement")
                .add("businessId", s.getId())
                .add("no", s.getNo())
                .add("name",s.getName())
                .ok ("操作成功");
    }

    /**
     * 根据id修改文件补充状态status
     */
    @ApiOperation(value = "根据id修改文件补充状态status")
    @PostMapping(value = "updateFileStatusById")
    public void updateFileStatusById(@RequestBody ConsultancyRetureListDto dto) {
        service.updateFileStatusById(dto);
    }


    /**
     * 修改数电发票报销数据(对老数据进行调整)
     */
    @ApiOperation(value = "修改数电发票报销数据(对老数据进行调整)")
    @GetMapping(value = "updateOldData")
    public void updateOldData(){
        service.updateOldData();
    }
}
