package com.jeeplus.consultancy.approvalType.controller;
import com.jeeplus.consultancy.approvalType.domain.ConsultancyReimbursementTypeInfo;
import com.jeeplus.consultancy.approvalType.mapper.ConsultancyReimbursementTypeMapper;
import com.jeeplus.consultancy.approvalType.service.ConsultancyReimbursementTypeForTreeDataService;
import com.jeeplus.consultancy.approvalType.service.ConsultancyReimbursementTypeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author: 王强
 * @create: 2022-11-25 09:18
 **/
@RestController
@Api(tags ="咨询-报销类型管理")
@RequestMapping(value = "/consultancyReimbursement/type")
public class ConsultancyReimbursementTypeController {

    @Resource
    private ConsultancyReimbursementTypeService service;

    @Resource
    private ConsultancyReimbursementTypeForTreeDataService treeService;

    @Resource
    private ConsultancyReimbursementTypeMapper mapper;

    /**
     * 列表查询
     * @param info
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/list")
    public ResponseEntity<List<ConsultancyReimbursementTypeInfo>> list(ConsultancyReimbursementTypeInfo info) {
        List<ConsultancyReimbursementTypeInfo> list = service.list(info);
        return ResponseEntity.ok(list);
    }

    /**
     * 列表查询
     * @param info
     * @return
     */
    @ApiOperation(value = "列表查询")
    @GetMapping("/bxList")
    public ResponseEntity<List<ConsultancyReimbursementTypeInfo>> bxList(ConsultancyReimbursementTypeInfo info) {
        List<ConsultancyReimbursementTypeInfo> list = service.bxList(info);
        return ResponseEntity.ok(list);
    }

    /**
     * 采购类型列表查询
     * @param info
     * @return
     */
    @ApiOperation(value = "采购类型列表查询")
    @GetMapping("/cgList")
    public ResponseEntity<List<ConsultancyReimbursementTypeInfo>> cgList(ConsultancyReimbursementTypeInfo info) {
        List<ConsultancyReimbursementTypeInfo> list = service.cgList(info);
        return ResponseEntity.ok(list);
    }

    /**
     * 新增/修改
     * @param info
     * @return
     */
    @ApiOperation(value = "新增/修改")
    @PostMapping("/save")
    public ResponseEntity<String> save(@RequestBody ConsultancyReimbursementTypeInfo info) {
        String s = service.save(info);
        return ResponseEntity.ok(s);
    }

    /**
     * 根据id查询
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id查询")
    @GetMapping("/findById")
    public ResponseEntity<ConsultancyReimbursementTypeInfo> findById(String id) {
        ConsultancyReimbursementTypeInfo info = mapper.selectById(id);
        return ResponseEntity.ok(info);
    }

    /**
     * 根据id删除
     * @param id
     * @return
     */
    @ApiOperation(value = "根据id删除")
    @GetMapping("/deleteById")
    public ResponseEntity<String> deleteById(String id) {
        mapper.deleteById(id);
        return ResponseEntity.ok("操作完成");
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/treeData")
    public ResponseEntity<List<ConsultancyReimbursementTypeInfo>> treeData(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<ConsultancyReimbursementTypeInfo> infos = treeService.treeDataForType(extId, type);
        return ResponseEntity.ok(infos);
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/treeData1")
    public ResponseEntity<List<ConsultancyReimbursementTypeInfo>> treeData1(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<ConsultancyReimbursementTypeInfo> infos = treeService.treeDataForType1(extId, type);
        return ResponseEntity.ok(infos);
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/summaryTreeData")
    public ResponseEntity<List<ConsultancyReimbursementTypeInfo>> summaryTreeData(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<ConsultancyReimbursementTypeInfo> infos = treeService.summaryTreeData(extId, type);
        return ResponseEntity.ok(infos);
    }

    /**
     * 查询树形
     * @param extId 排除的ID
     * @return
     */
    @ApiOperation(value = "查询树形")
    @GetMapping("/treeData2")
    public ResponseEntity<List<ConsultancyReimbursementTypeInfo>> treeData2(@RequestParam(required = false) String extId, @RequestParam(required = false) String type) throws Exception{
        List<ConsultancyReimbursementTypeInfo> infos = treeService.treeDataForType2(extId, type);
        return ResponseEntity.ok(infos);
    }

}
